console.log('Minty Woo Mini Cart JS loaded');

class MiniCart {
    constructor() {
        this.cartElement = document.getElementById('minty-woo-mini-cart');
        this.cartItemsContainer = document.querySelector('.minty-cart-items');
        this.overlayElement = document.getElementById('overlay');
        this.mintyCartApi = new MintyWooCartApi();
        this.toggleCartElements = [
            this.overlayElement,
            document.getElementById('minty-woo-mini-cart-button'),
            document.getElementById('minty-woo-mini-cart-close')
        ];

        // Load cart items for the first time
        this.cartItems = MintyWooMiniCartVars.CartItems;
        this.totalPrice = 0;
        this.loadCartItems(false);
        this.init();
    }

    async loadCartItems(loadFromApi = true) {
        if (loadFromApi) {
            const cart = await this.mintyCartApi.getLatestCartData();
            this.cartItems = cart.context;
        }

        if (this.isCartEmpty()) {
            return;
        }

        // Get the template and the container where the cart items will be placed
        const template = document.getElementById('cart-item-template').content;

        // Loop through the cart items and update the DOM
        this.totalPrice = 0;
        this.cartItems.forEach(item => {
            this.totalPrice += item.price * item.quantity;
            const existingCartItem = document.getElementById(`cart-item-${item.key}`);
            // Cart item already exists, update the quantity and price
            if (existingCartItem) {
                existingCartItem.querySelector('.minty-cart-product-quantity').textContent = item.quantity;
                existingCartItem.querySelector('.product-price-amount').textContent = item.price;
                return;
            }

            // The item does not exist in the DOM, create a new one
            const clone = document.importNode(template, true);
            clone.querySelector('.minty-cart-item').id = `cart-item-${item.key}`;
            clone.querySelector('.minty-cart-product-image').src = item.image_url;
            clone.querySelector('.minty-cart-product-image').alt = item.title;
            clone.querySelector('.minty-cart-product-name').textContent = item.title;
            clone.querySelector('.product-price-amount').textContent = item.price;
            clone.querySelector('.minty-cart-product-quantity').textContent = item.quantity;

            // Append the new item to the container
            this.cartItemsContainer.appendChild(clone);
        });

        // Delete items that are no longer in the cart
        const existingItems = this.cartItemsContainer.querySelectorAll('.minty-cart-item');
        existingItems.forEach(existingItem => {
            const cartKey = existingItem.id.replace('cart-item-', '');
            const itemExistsInCart = this.cartItems.some(item => item.key === cartKey);

            if (!itemExistsInCart) {
                existingItem.remove();
            }
        });

        // Set the event listeners for the quantity buttons
        this.updateTotalPrice();
        this.setCartItemListeners();
    }


    init() {
        // Event listeners voor het openen en sluiten van de winkelwagen
        this.toggleCartElements.forEach(element => {
            element.addEventListener('click', this.toggleCart.bind(this));
        });

        // TODO: Check if this works with loading more products on one page
        // Triggers for adding products to the cart, this will update the mini cart
        const addToCartTriggers = document.querySelectorAll('.add_to_cart_button, .ajax_add_to_cart');
        addToCartTriggers.forEach(button => {
            button.addEventListener('click', function (event) {
                event.preventDefault();

                // Keep checking if button has loading class, when it's removed we know the product has been added
                const checkIfLoadingIsRemoved = setInterval(() => {
                    if (!button.classList.contains('loading')) {
                        clearInterval(checkIfLoadingIsRemoved);
                        this.loadCartItems(); // Update the cart with the latest data
                    }
                }, 100);
            }.bind(this));
        });

        this.updateTotalPrice();
    }

    isCartEmpty() {
        if (this.cartItems.length > 0) {
            const emptyMessage = this.cartItemsContainer.querySelector('.cart-empty-message');
            if (emptyMessage) {
                emptyMessage.remove();
            }
            return false;
        }

        const emptyTemplate = document.getElementById('cart-empty-template').content.cloneNode(true);
        this.cartItemsContainer.appendChild(emptyTemplate);
        return true;
    }


    // Set event listeners for the buttons in a cart item
    setCartItemListeners() {
        // Delete item from cart button
        const removeCartItemButtons = this.cartElement.querySelectorAll('.minty-cart-remove-item-button');
        removeCartItemButtons.forEach(button => {
            if (!button.dataset.listenerAdded) {
                button.addEventListener('click', this.removeCartItem.bind(this, button));
                button.dataset.listenerAdded = 'true';
            }
        });

        // Increase / decrease cart item quantity buttons
        const quantityButtons = document.querySelectorAll('.minty-cart-quantity-button');
        quantityButtons.forEach(button => {
            if (!button.dataset.listenerAdded) {
                const isIncrease = button.classList.contains('increase');
                const change = isIncrease ? 1 : -1;
                button.addEventListener('click', this.updateQuantity.bind(this, button, change));
                button.dataset.listenerAdded = 'true';
            }
        });
    }

    removeCartItem(button) {
        const product = button.closest('.minty-cart-item');
        const cartKeyId = product.id;
        const cartKey = cartKeyId.replace('cart-item-', '');
        product.classList.add('removing');

        // Search for the item in the cartItems array
        const itemIndex = this.cartItems.findIndex(item => item.key === cartKey);
        if (itemIndex === -1) {
            console.error('Item not found in cartItems');
            return;
        }

        const item = this.cartItems[itemIndex];


        setTimeout(() => {
            this.mintyCartApi.RemoveCartItem(cartKey)
                .then(() => {
                    product.remove();

                    // Delete the item from the cartItems array
                    this.cartItems.splice(itemIndex, 1);

                   // Update total price after removing the item
                    this.totalPrice -= (item.quantity * item.price);
                    this.updateTotalPrice();

                    // Check if the cart is empty
                    if (this.isCartEmpty()) {
                        this.toggleCart();
                    }
                })
                .catch(error => {
                    console.error('Failed to remove item:', error);
                    product.classList.remove('removing');
                });
        }, 300); // Match the CSS transition duration
    }

    updateTotalPrice() {
        const totalPriceElement = this.cartElement.querySelector('.minty-cart-total-amount');
        // Format the total price to always have 2 decimal places and use a comma as the decimal separator
        totalPriceElement.textContent = this.totalPrice.toLocaleString('nl-NL', { minimumFractionDigits: 2, maximumFractionDigits: 2 });
    }

    updateQuantity(button, change) {
        const quantityElement = button.closest('.minty-cart-product-quantity-controls').querySelector('.minty-cart-product-quantity');
        const cartKeyId = button.closest('.minty-cart-item').id;
        const cartKey = cartKeyId.replace('cart-item-', '');
        let currentQuantity = parseInt(quantityElement.textContent);

        // Update the quantity
        currentQuantity += change;

        // Prevent quantity from going below 1
        if (currentQuantity < 1) {
            currentQuantity = 1;
        }

        // Update the quantity in the DOM
        quantityElement.textContent = currentQuantity;

        // Find the corresponding item in the cartItems array
        const item = this.cartItems.find(item => item.key === cartKey);
        if (item) {
            // Calculate the difference in quantity
            const quantityDifference = currentQuantity - item.quantity;

            // Update the quantity in the item
            item.quantity = currentQuantity;

            // Update the total price
            this.totalPrice += quantityDifference * item.price;
            this.updateTotalPrice();
        }

        // Clear any previous timeout to debounce the update call
        clearTimeout(this.updateTimeout);

        // Set a one second timeout to debounce the update call
        this.updateTimeout = setTimeout(() => {
            this.mintyCartApi.updateCartItemQuantity(cartKey, currentQuantity);
        }, 1000);
    }

    toggleCart() {
        const isHidden = this.cartElement.classList.contains('hidden');

        if (isHidden) {
            this.cartElement.classList.remove('hidden');
            this.overlayElement.classList.remove('hidden');

            setTimeout(() => {
                this.cartElement.classList.add('active');
                this.overlayElement.classList.add('active');
            }, 10);
            return;
        }

        this.cartElement.classList.remove('active');
        this.overlayElement.classList.remove('active');

        setTimeout(() => {
            this.cartElement.classList.add('hidden');
            this.overlayElement.classList.add('hidden');
        }, 300); // Match the CSS transition duration
    }
}

document.addEventListener('DOMContentLoaded', () => {
    new MiniCart();
});

