<?php

namespace MintySupport\Classes\Core\Api;


use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

abstract class RestApi {
	private ?Client $apiClient = null;
	private array $headers;
	private bool $debug;

	public function __construct( $debug = false ) {
		$this->headers = $this->getDefaultHeaders();
		$this->debug   = $debug;
	}

	/**
	 * @param string $url
	 * @param bool $rawResponse
	 *
	 * @return bool|array|object|string
	 */
	public function getRequest( string $url, bool $rawResponse = false ): array|object|string|bool {
		try {
			$response = $this->getApiClient()->get( $url, [
				'headers' => $this->getHeaders()
			] );
		} catch ( GuzzleException $e ) {
			$this->onError( $e, $url );
			if ( $this->debug ) {
				$this->debugErrorMessage( $e->getMessage() );
			}

			$this->resetHeaders();

			return false;
		}

		$this->resetHeaders();

		$resBody = $response->getBody()->getContents();

		return $rawResponse === true ? $resBody : json_decode( $resBody );
	}

	/**
	 * @param string $url
	 * @param $body
	 *
	 * @return bool|array|object
	 */
	public function postRequest( string $url, $body ): bool|array|object {
		try {
			// Update the content type to application/x-www-form-urlencoded
			$this->addHeader( [ "Content-Type" => "application/x-www-form-urlencoded" ] );
			$response = $this->getApiClient()->post( $url, [
				'headers'     => $this->getHeaders(),
				'form_params' => $body // Use form_params instead of body for URL encoding
			] );
		} catch ( GuzzleException $e ) {
			$this->onError( $e, $url, $body );
			if ( $this->debug ) {
				$this->debugErrorMessage( $e->getMessage() );
			}

			$this->resetHeaders();

			return false;
		}
		$this->resetHeaders();
		$response = json_decode( $response->getBody()->getContents(), true );

		return $response ?? false;
	}

	/**
	 * @param string $url
	 * @param array|object|string $body
	 *
	 * @return bool|array|object
	 */
	public function putRequest( string $url, array|object|string $body ): bool|array|object {
		try {
			$this->addHeader( [ "Content-Type" => "application/json" ] );
			$response = $this->getApiClient()->put( $url, [
				'headers' => $this->getHeaders(),
				'body'    => ( is_array( $body ) || is_object( $body ) ) ? json_encode( $body ) : $body
			] );

		} catch ( GuzzleException $e ) {
			$this->onError( $e, $url, $body );
			if ( $this->debug ) {
				$this->debugErrorMessage( $e->getMessage() );
			}
			$this->resetHeaders();

			return false;
		}
		$this->resetHeaders();
		$response = json_decode( $response->getBody()->getContents() );

		return $response ?? false;
	}

	/**
	 * @param string $url
	 *
	 * @return bool
	 */
	public function deleteRequest( string $url ): bool {
		try {
			$this->getApiClient()->delete( $url, [
				'headers' => $this->getHeaders()
			] );
		} catch ( GuzzleException $e ) {
			$this->onError( $e, $url );
			if ( $this->debug ) {
				$this->debugErrorMessage( $e->getMessage() );
			}

			return false;
		}

		return true;
	}

	public function getApiClient(): ?Client {
		return $this->apiClient ?? ( $this->apiClient = new Client(
			[
				'base_uri' => $this->getBaseUrl(),
				'verify'   => false,
			]
		) );
	}


	public function getDefaultHeaders(): array {
		return [];
	}

	/**
	 * @return array
	 */
	public function getHeaders(): array {
		return $this->headers;
	}

	private function resetHeaders(): void {
		$this->headers = $this->getDefaultHeaders();
	}

	/**
	 * @param array $headers
	 */
	public function addHeader( array $headers ): void {
		$this->headers = array_merge( $this->headers, $headers );
	}

	private function debugErrorMessage(string $error): void {
		$errorData = json_decode($error, true);
		$isJsonValid = json_last_error() === JSON_ERROR_NONE && isset($errorData["error"]["code"]);

		if (!$isJsonValid) {
			if ( str_contains( $error, "InvalidAuthenticationToken" ) ) {
				$this->setErrorAndLog("De ingelogde gebruiker heeft niet de juiste rechte tot deze microsoft azure applicatie.");
				return;
			}
			// Als de JSON-decoderingsfunctie niet succesvol was en het bevat geen specifieke term, gebruik dan de originele fout
			$this->setErrorAndLog($error);
			return;
		}

		$errorCode = $errorData["error"]["code"];
		if ($errorCode === "InvalidAuthenticationToken") {
			$this->setErrorAndLog("De ingelogde gebruiker heeft niet de juiste rechte tot deze microsoft azure applicatie.");
			return;
		}

		$errorMessage = $errorData["error"]["message"];
		$this->setErrorAndLog($errorMessage);
	}

	private function setErrorAndLog(string $message): void {
		error_log($message);
		$_SESSION["minty_mail_error_message"] = $message;
	}

	/**
	 * Returns the api BseUrl
	 * @return string
	 */
	abstract public function getBaseUrl(): string;

	public function onError( GuzzleException $e, string $url, mixed $body = [] ): void {
	}
}
