<?php

namespace MintySupport\Classes\Core\Endpoints;

use MintySupport\Classes\Core\Endpoints\Response;

abstract class AjaxEndpoint
{

	private ?Response $errorRes = null;
	private ?Response $successRes = null;
	private mixed $res = null;

	protected static string $Post = "POST";
	protected static string $Get = "GET";
	protected static string $Put = "PUT";

	abstract public static function getRouteName(): string;
	abstract public static function getRequestType(): string;
	abstract public function handle(object|array $requestData): void;

	/**
	 * @return void
	 */
	public function init(): void
	{
		add_action( "wp_ajax_nopriv_{$this::getRouteName()}", [$this, "execute"]);
		add_action("wp_ajax_{$this::getRouteName()}",[$this, "execute"]);
	}

	/**
	 * @return void
	 */
	public function execute(): void
	{
		header('Content-Type: application/json');

		if ($_SERVER['REQUEST_METHOD'] !== $this::getRequestType()) {
			$this->setErrorRes(405,
				"Wrong request Type. Incoming {$_SERVER['REQUEST_METHOD']} wanting {$this::getRequestType()}");
		}

		$requestData = $this->getRequestData();

		$this->handle($requestData);

		$this->emitResponse();
	}

	/**
	 * Get the request data based on the request type.
	 *
	 * @return array|object The request data.
	 */
	private function getRequestData(): array|object
	{
		$requestData = [];
		if ($this::getRequestType() === $this::$Post) {
			$phpData = file_get_contents('php://input');


			if($phpData)
				$requestData = json_decode($phpData, true);
			else
				$this->setErrorRes(400, "No request body");
		} elseif ($this::getRequestType() === $this::$Get) {
			$requestData = $_GET ?? [];
		}

		return $requestData;
	}

	/**
	 * @return Response|null
	 */
	private function getSuccessRes(): ?Response
	{
		if ($this->successRes === null) {
			$this->successRes = new Response();
		}

		return $this->successRes;
	}

	/**
	 * @param string $message
	 * @param string $title
	 * @param mixed $context
	 * @return void
	 */
	public function setSuccessRes(string $message, string $title = "Success", mixed $context = []): void
	{
		if ($this->successRes === null)
			$this->successRes = new Response();

		$this->successRes->title = $title;
		$this->successRes->message = $message;
		$this->successRes->context = $context;

		$this->emitResponse($this->successRes);
	}

	public function getResponse(): mixed
	{
		return $this->res;
	}

	public function setResponse(mixed $res): void
	{
		$this->res = $res;
	}

	/**
	 * @param int $statusCode
	 * @param string $message
	 * @param string $title
	 * @param mixed $context
	 * @return never
	 */
	public function setErrorRes(int $statusCode, string $message, string $title = "Error", mixed $context = []): never
	{
		http_response_code($statusCode);

		if ($this->errorRes === null)
			$this->errorRes = new Response();

		$this->errorRes->title = $title;
		$this->errorRes->message = $message;
		$this->errorRes->context = $context;

		$this->emitResponse($this->errorRes);
	}

	/**
	 * @param Response|null $response
	 * @return never
	 */

	private function emitResponse(?Response $response = null): never
	{
		if ($response === null) {
			$response = !empty($this->getResponse()) ? $this->getResponse() : $this->getSuccessRes();
		}

//		echo json_encode( $response, JSON_THROW_ON_ERROR );
		die();
	}

	/**
	 * The URL for the admin-ajax.php file with the specified action.
	 *
	 * @return string
	 */
	public static function getUrl(): string
	{
		return admin_url('admin-ajax.php') . "?action=" . static::getRouteName();
	}
}
