<?php

namespace MintySupport\Classes\Modules\SmtpMail\AdminPages;

use Google\Client as Google_Client;
use MintySupport\Classes\Core\Managers\EncryptManager;
use MintySupport\Classes\Core\Managers\SettingsManager;
use MintySupport\Classes\Core\Settings\BaseAdminSettingsPage;
use MintySupport\Classes\Modules\SmtpMail\MailProviders\MicrosoftProvider;
use MintySupport\Classes\Modules\SmtpMail\RestApis\Microsoft\MicrosoftGraphRestApi;


class SmtpMailBase extends BaseAdminSettingsPage {
	public function getName(): string {
		return 'smtpmail';
	}

	public function getSettingName(): string {
		return 'smtp_mail_enabled';
	}

	public function getDisplayName(): string {
		return 'SMTP Mail';
	}

	/**
	 * @throws \Exception
	 */
	public function onLoad(): array {
		// Laad de instellingen uit de database
		$settingsServices = SettingsManager::getInstance();
		$encryptServices  = EncryptManager::getInstance();
		$redirectUri   = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http' ) . '://' . $_SERVER['HTTP_HOST'] . '/wp-admin/';

		$microsoftMailApplicationId = $settingsServices->getSettings( 'microsoft_mail_application_id', false );
		$microsoftMailAccessToken   = $settingsServices->getSettings( 'microsoft_mail_access_token', false );
		$microsoftMailUserMail     = $settingsServices->getSettings( 'microsoft_mail_user_mail', false );

		if ( $microsoftMailAccessToken && !$microsoftMailUserMail ) {
			$graphApi = new MicrosoftGraphRestApi();
			$microsoftUserInfo = $graphApi->getUserProfile();
			$settingsServices->setSetting( 'microsoft_mail_user_mail', $microsoftUserInfo["mail"] );
			$settingsServices->setSetting( 'microsoft_mail_user_name', $microsoftUserInfo["displayName"]);
			$settingsServices->save();
		}

		return [
			// Algemeen
			"providers"                        => [ 'standaard', 'microsoft', 'google' ],
			"mail_sender_name"                 => $settingsServices->getSettings( 'mail_sender_name', false ),
			"mail_sender_email"                => $settingsServices->getSettings( 'mail_sender_email', false ),
			"force_mail_sender_name"           => $settingsServices->getSettings( 'force_mail_sender_name', false ),
			"force_mail_sender_email"          => $settingsServices->getSettings( 'force_mail_sender_email', false ),
			"smtp_mail_enabled"                => $settingsServices->getSettings( 'smtp_mail_enabled', false ),
			"smtp_mail_provider"               => $settingsServices->getSettings( 'smtp_mail_provider', false ),
			"smtp_mail_tab_selected"           => $settingsServices->getSettings( 'smtp_mail_tab_selected', false ),

			// Google
			"google_client"                    => new Google_Client(),
			"google_client_id"                 => $settingsServices->getSettings( 'google_client_id', false ),
			"google_client_secret"             => $settingsServices->getSettings( 'google_client_secret', false ),
			"google_redirect_uri"              => $redirectUri,
			"google_access_token"              => $settingsServices->getSettings( 'google_access_token', false ),
			"google_client_username"           => $settingsServices->getSettings( 'google_client_username', false ),
			"google_client_fullname"           => $settingsServices->getSettings( 'google_client_fullname', false ),

			// Microsoft
			"microsoftMailApplicationId"       => $settingsServices->getSettings( 'microsoft_mail_application_id', false ),
			"microsoftMailApplicationPassword" => $settingsServices->getSettings( 'microsoft_mail_application_password', false ),
			"microsoftMailRedirectUri"         => $redirectUri,
			"microsoftMailUserMail"            => $settingsServices->getSettings( 'microsoft_mail_user_mail', false ),
			"microsoftMailUserName"            => $settingsServices->getSettings( 'microsoft_mail_user_name', false ),
			"microsoftMailAccessToken"         => $settingsServices->getSettings( 'microsoft_mail_access_token', false ),
			"microsoftMailAuthUrl"             => "https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id=$microsoftMailApplicationId&response_type=code&redirect_uri=$redirectUri&scope=openid+offline_access+Mail.Send+Mail.Send.Shared&state=" . $this->getState(),

			// Standaard / SMTP
			"mail_host"                        => $settingsServices->getSettings( 'mail_host', false ),
			"mail_port"                        => $settingsServices->getSettings( 'mail_port', false ),
			"mail_security"                    => $settingsServices->getSettings( 'mail_security', false ),
			"mail_username"                    => $settingsServices->getSettings( 'mail_username', false ),
			"mail_password"                    => $encryptServices->decryptData( $settingsServices->getSettings( 'mail_password', false ) ),
		];
	}

	protected function getPostedSettings(): array {
		return [
			// Algemeen
			'mail_sender_name',
			'mail_sender_email',
			'force_mail_sender_name',
			'force_mail_sender_email',
			'smtp_mail_enabled',
			'smtp_mail_provider',
			'smtp_mail_tab_selected',

			// Microsoft
			'microsoft_mail_application_id',
			'microsoft_mail_application_password',
			'microsoft_mail_redirect_uri',

			// Standaard / SMTP
			'mail_host',
			'mail_port',
			'mail_security',
			'mail_username',
			'mail_password',

			// Google
			'google_client_id',
			'google_client_secret',
			'google_redirect_uri',
		];
	}

	private function getState(): string {
		$nonce = 'minty-support-smtp-' . wp_create_nonce( 'minty_support_smtp_nonce' );
		update_user_meta( get_current_user_id(), 'graph_api_state', $nonce );

		return $nonce;
	}

	public function getOnSaveTrigger(): bool {
		return ( ! empty( $_POST ) && isset( $_POST['action'] ) && ( $_POST['action'] === 'save_minty_mail_settings' || $_POST['action'] === 'send_minty_test_mail' || $_POST['action'] === 'remove_minty_client_tokens' || $_POST['action'] === 'save_minty_support_settings' || $_POST['action'] === 'save_minty_google_settings' ) );
	}
}