<?php

namespace MintySupport\Classes\Modules\SmtpMail\MailProviders;

use Exception;
use Google\Client as Google_Client;
use MintySupport\Classes\Core\Managers\SettingsManager;
use MintySupport\Classes\Core\Settings\MailProviders\OAuthBase;
use MintySupport\Classes\Core\Settings\MailProviders\ProviderBase;
use MintySupport\Classes\Modules\SmtpMail\PhpMailAdapters\GoogleMailAdapter;
use PHPMailer\PHPMailer\PHPMailer;

class GoogleProvider extends OAuthBase {

	/**
	 * @throws Exception
	 */
	public function handleOAuth(): void {
		$settingsData = SettingsManager::getInstance();
		$clientId     = $settingsData->getSettings( 'google_client_id', false );
		$clientSecret = $settingsData->getSettings( 'google_client_secret', false );
		$redirectUri  = $settingsData->getSettings( 'google_redirect_uri', false );
		$client = new Google_Client();
		$client->setClientId( $clientId );
		$client->setClientSecret( $clientSecret );
		$client->setRedirectUri( $redirectUri );
		$client->addScope( 'https://www.googleapis.com/auth/gmail.send' );
		$client->addScope( 'openid' );
		$client->addScope( 'email' );
		$client->addScope( 'profile' );
		$client->setAccessType( 'offline' ); // Voor het verkrijgen van een refresh token

		// Ruil de authorization code in voor access en refresh tokens
		$token = $client->fetchAccessTokenWithAuthCode( $_GET['code'] );

		// Controleer op fouten
		if ( array_key_exists( 'error', $token ) ) {
			throw new Exception( implode( ', ', $token ) );
		}

		// Haal het e-mailadres op uit het id_token
		if ( isset( $token['id_token'] ) ) {
			$idTokenPayload = explode( '.', $token['id_token'] )[1];
			$idTokenPayload = json_decode( base64_decode( $idTokenPayload ), true, 512, JSON_THROW_ON_ERROR );
			$settingsData->setSetting( 'google_client_username', $idTokenPayload['email'] );
			$settingsData->setSetting( 'google_client_fullname', $idTokenPayload['name'] );
			$settingsData->save();

		}

		// Sla het access token, refresh token en het e-mailadres op
		$accessToken  = $token['access_token'];
		$refreshToken = $token['refresh_token'];

		$this->saveTokens( $accessToken, $refreshToken);

		wp_redirect( $redirectUri . '/admin.php?page=minty-support&tab=smtpmail' );
		exit;
	}

	public function setup(  PHPMailer $phpmailer): void {
		$settingsData = SettingsManager::getInstance();
		$clientId     = $settingsData->getSettings( 'google_client_id', false );
		$clientSecret = $settingsData->getSettings( 'google_client_secret', false );
		$redirectUri  = $settingsData->getSettings( 'google_redirect_uri', false );
		$refreshToken = $settingsData->getSettings( 'google_refresh_token', false );
		$accessToken  = $settingsData->getSettings( 'google_access_token', false );

		if ( ! $clientId || ! $clientSecret || ! $refreshToken || ! $accessToken || ! $redirectUri ) {
			$_SESSION['minty_mail_error_message'] = 'Google OAuth2 instellingen ontbreken of zijn ongeldig.';

			return;
		}

		$client = new Google_Client();
		$client->setClientId( $clientId );
		$client->setClientSecret( $clientSecret );
		$client->setRedirectUri( $redirectUri );
		$client->addScope( 'https://www.googleapis.com/auth/gmail.send' );
		$client->setAccessType( 'offline' );
		$client->setAccessToken( $accessToken );

		// Refresh token if needed
		if ( $client->isAccessTokenExpired() ) {
			$newToken = $client->fetchAccessTokenWithRefreshToken( $refreshToken );
			if ( isset( $newToken['error'] ) ) {
				$_SESSION['minty_mail_error_message'] = 'Error getting new access token: ' . $newToken['error_description'];

				return;
			}

			$accessToken = $newToken['access_token'];

			// Store the new access token and possibly the refresh token
			$this->saveTokens( $accessToken, $refreshToken );
		}

		if ( ! $accessToken ) {
			$_SESSION['minty_mail_error_message'] = 'Kan geen geldig toegangstoken verkrijgen of het token is verlopen.';

			return;
		}

		new GoogleMailAdapter( $phpmailer, $accessToken );

	}

	public function saveTokens( string $accessToken, string $refreshToken ): void {
		$settingsData = SettingsManager::getInstance();
		$settingsData->setSetting( 'google_access_token', $accessToken );
		$settingsData->setSetting( 'google_refresh_token', $refreshToken );
		$settingsData->save();

	}

	public function removeTokens(): void {
		$settingsServices = SettingsManager::getInstance();
		$settingsServices->setSetting( 'google_access_token', '' );
		$settingsServices->setSetting( 'google_refresh_token', '' );
		$settingsServices->setSetting( 'google_client_username', '' );
		$settingsServices->setSetting( 'google_client_fullname', '' );
		$settingsServices->save();
	}
}