<?php

namespace MintySupport\Classes\Modules\SmtpMail\MailProviders;

use MintySupport\Classes\Core\Managers\SettingsManager;
use MintySupport\Classes\Core\Settings\MailProviders\OAuthBase;
use MintySupport\Classes\Modules\SmtpMail\PhpMailAdapters\GraphMailAdapter;
use MintySupport\Classes\Modules\SmtpMail\RestApis\Microsoft\MicrosoftGraphRestApi;
use MintySupport\Classes\Modules\SmtpMail\RestApis\Microsoft\OAuthRestApi;
use PHPMailer\PHPMailer\PHPMailer;

class MicrosoftProvider extends OAuthBase {
	public function handleOAuth(): void {
		if ( ! isset( $_GET['state'] ) ) {
			wp_die( 'Ongeldige OAuth state. Probeer het opnieuw of neem contact op met de support.', 'OAuth Error' );
		}
		$settingsData  = SettingsManager::getInstance();
		$clientId      = $settingsData->getSettings( 'microsoft_mail_application_id', false );
		$returnedState = $_GET['state'];
		$clientSecret  = $settingsData->getSettings( 'microsoft_mail_application_password', false );
		$redirectUri   = $settingsData->getSettings( 'microsoft_mail_redirect_uri', false );

		$expectedState = get_user_meta( get_current_user_id(), 'graph_api_state', true );

		// Vergelijk de verwachte state met de geretourneerde state
		if ( $returnedState !== $expectedState ) {
			wp_die( 'Ongeldige OAuth state. Probeer het opnieuw of neem contact op met de support.', 'OAuth Error' );
		}

		$code     = $_GET['code'];
		$oAuthApi = new OAuthRestApi();
		$tokens   = $oAuthApi->getAccessToken( $clientId, $clientSecret, $code, $redirectUri );
		$this->saveTokens( $tokens['access_token'], $tokens['refresh_token'] );
		$graphApi = new MicrosoftGraphRestApi();
		$microsoftUserInfo = $graphApi->getUserProfile();

		$settingsData->setSetting( 'microsoft_mail_user_mail', $microsoftUserInfo["mail"] );
		$settingsData->setSetting( 'microsoft_mail_user_name', $microsoftUserInfo["displayName"]);
		$settingsData->save();
		wp_redirect( $redirectUri . '/admin.php?page=minty-support&tab=smtpmail' );
		exit;
	}

	public function setup( PHPMailer $phpmailer ): void {
		$settingsData = SettingsManager::getInstance();
		$accessToken  = $settingsData->getSettings( 'microsoft_mail_access_token', false );
		$refreshToken = $settingsData->getSettings( 'microsoft_mail_refresh_token', false );
		$clientId     = $settingsData->getSettings( 'microsoft_mail_application_id', false );
		$clientSecret = $settingsData->getSettings( 'microsoft_mail_application_password', false );

		if ( ! $accessToken ) {
			error_log( 'Geen geldig access token beschikbaar voor Microsoft Graph API.' );

			return;
		}

		// Controleer de geldigheid van het access token
		$oAuthApi  = new OAuthRestApi();
		$tokenInfo = $oAuthApi->validateAccessToken( $accessToken );
		if ( $tokenInfo->isValid ) {
			new GraphMailAdapter( $phpmailer, $accessToken, $settingsData );

			return;
		}

		// Access token is ongeldig of verlopen, vraag een nieuwe aan met de refresh token
		if ( ! $refreshToken ) {
			error_log( 'Geen refresh token beschikbaar om een nieuw access token aan te vragen.' );

			return;
		}

		$newTokens = $oAuthApi->getRefreshToken( $refreshToken, $clientId, $clientSecret );
		if ( ! $newTokens ) {
			error_log( 'Kon geen nieuw access token verkrijgen met de refresh token.' );

			return;
		}

		// Sla het nieuwe access token en refresh token op in settings
		$accessToken = $newTokens['access_token'];
		$this->saveTokens( $accessToken, $newTokens['refresh_token'] );

		new GraphMailAdapter( $phpmailer, $accessToken, $settingsData );
	}

	public function saveTokens( string $accessToken, string $refreshToken ): void {
		$settings = SettingsManager::getInstance();
		$settings->setSetting( 'microsoft_mail_access_token', $accessToken );
		$settings->setSetting( 'microsoft_mail_refresh_token', $refreshToken );
		$settings->save();
	}

	public function removeTokens(): void {
		$settingsServices = SettingsManager::getInstance();

		$settingsServices->setSetting( 'microsoft_mail_access_token', '' );
		$settingsServices->setSetting( 'microsoft_mail_refresh_token', '' );
		$settingsServices->setSetting( 'microsoft_mail_user_mail', '' );
		$settingsServices->setSetting( 'microsoft_mail_user_name', '' );

		$settingsServices->save();
	}
}