<?php

namespace MintySupport\Classes\Modules\SmtpMail\PhpMailAdapters;

use JsonException;
use MintySupport\Classes\Core\Managers\SettingsManager;
use PHPMailer\PHPMailer\Exception;
use PHPMailer\PHPMailer\PHPMailer;

class GraphMailAdapter {
	private string $accessToken;
	private PHPMailer $phpMailer;

	/**
	 * @throws JsonException
	 */
	public function __construct(PHPMailer $phpMailer, $accessToken) {
		$this->phpMailer   = $phpMailer;
		$this->accessToken = $accessToken;
		$this->overridePhpMailer();
		$this->sendEmailUsingGraphAPI();
	}

	/**
	 * @throws Exception
	 */
	private function overridePhpMailer(): void {
		$this->phpMailer->isSMTP();
		$this->phpMailer->Sender   = '';
		$settingsManager = SettingsManager::getInstance();
		$fromEmail = $settingsManager->getSettings( 'microsoft_mail_user_mail', false );
		if ($settingsManager->getSettings('force_mail_sender_name') === 'true') {
			$fromName = $settingsManager->getSettings('mail_sender_name');
		} else {
			$fromName = $settingsManager->getSettings( 'microsoft_mail_user_name', false );
		}
		$this->phpMailer->From     = $fromEmail;
		$this->phpMailer->FromName = $fromName;
		$this->phpMailer->setFrom($fromEmail, $fromName);
	}

	public function sendEmailUsingGraphAPI(): void {
		$url     = 'https://graph.microsoft.com/v1.0/me/sendMail';
		$headers = [
			'Authorization' => 'Bearer ' . $this->accessToken,
			'Content-Type'  => 'application/json',
			'Accept'        => 'application/json'
		];

		$body = $this->formatMailData();

		$response = wp_remote_post($url, [
			'headers'     => $headers,
			'body'        => json_encode($body, JSON_THROW_ON_ERROR),
			'method'      => 'POST',
			'data_format' => 'body'
		]);

		$_SESSION['has_email_send_minty'] = true;


		if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
			$error = is_wp_error($response) ? $response->get_error_message() : wp_remote_retrieve_body($response);
			if (!$error) {
				return;
			}

			// Probeer de foutmelding te decoderen als JSON
			$errorData = json_decode($error, true, 512, JSON_THROW_ON_ERROR);

			// Controleer of de JSON-decoderingsfunctie succesvol was en de foutboodschap bevat
			if (json_last_error() === JSON_ERROR_NONE && isset($errorData['error']['message'])) {
				$errorMessage = $errorData['error']['message'];
			} else {
				// Als de JSON-decoderingsfunctie niet succesvol was, gebruik dan de originele fout
				$errorMessage = $error;
			}

			// Sla de foutboodschap op in de sessie
			$_SESSION['minty_mail_error_message'] = $errorMessage;
			$_SESSION['has_email_send_minty'] = false;
			error_log('Error: ' . $errorMessage);
		}
	}

	private function formatMailData(): array {
		return [
			'message'         => [
				'subject'      => $this->phpMailer->Subject,
				'body'         => [
					'contentType' => 'HTML',
					'content'     => $this->phpMailer->Body
				],
				'from' => [
					'emailAddress' => [
						'address' => $this->phpMailer->From,
						'name'    => $this->phpMailer->FromName,
					]
				],
				'toRecipients' => array_map(function ($address) {
					return ['emailAddress' => ['address' => $address[0]]];
				}, $this->phpMailer->getToAddresses()),
				'attachments'  => $this->getAttachments()
			],
			'saveToSentItems' => 'true',
		];
	}

	private function getAttachments(): array {
		$attachments = [];

		// Doorloop alle bijlagen die zijn ingesteld via PHPMailer
		foreach ($this->phpMailer->getAttachments() as $attachment) {
			$fileContent = file_get_contents($attachment[0]);
			$base64File  = base64_encode($fileContent);

			$attachments[] = [
				'@odata.type'   => '#microsoft.graph.fileAttachment',
				'name'          => $attachment[2],
				'contentBytes'  => $base64File,
				'contentType'   => $attachment[4],
			];
		}

		return $attachments;
	}
}
